@@ -1,14 +1,19 @@
 optimr <- function(par, fn, gr=NULL, hess=NULL, method=NULL, lower=-Inf, upper=Inf, 
             hessian=FALSE, control=list(), ...) {
-
-  fname <- as.list(sys.call())$fn # FAILED rlang::as_name(as.list(sys.call())$fn)
+  parOK<-is.vector(par, mode="double")
+  if (! parOK) {
+     warning("optimr: par is NOT a vector of type double -- coercing")
+     par <- as.vector(as.double(par))
+  }
+  npar <- length(par)
+  parnam<-names(par)
+  fname <- as.list(sys.call())$fn # Following FAILED rlang::as_name(as.list(sys.call())$fn)
   if (length(method) > 1) stop("optimr requires single method")
   if (is.null(method)) method <- control$defmethod # Set a default method
   fn1 <- function(par) fn(par,...) # avoid dotarg clashes
   gr1 <- if (!is.null(gr) && !is.character(gr)) function(par) gr(par,...) # ?? will this fail for quoted names
   he1 <- if (!is.null(hess) && !is.character(hess)) function(par) hess(par,...) 
 
-  npar <- length(par)
   ctrl <- ctrldefault(npar)
   ncontrol <- names(control)
   nctrl <- names(ctrl)
@@ -54,6 +59,7 @@ optimr <- function(par, fn, gr=NULL, hess=NULL, method=NULL, lower=-Inf, upper=I
 	  ans$counts[2] <- NA # save function and gradient count information
 	  ans$message <- paste("Missing method ",method)
     ans$hessian <- NULL
+    names(ans$par) <- parnam # restore the names
     return(ans) # can't proceed without solver
   }
 
@@ -68,6 +74,7 @@ optimr <- function(par, fn, gr=NULL, hess=NULL, method=NULL, lower=-Inf, upper=I
 #      par <- bdmsk$bvec
 #  }
   control$have.bounds <- bdmsk$bounds # and set a control value
+  if (control$have.bounds && !(method %in% control$bdmeth)) stop("Bounded problem with unsuitable method")
 
   orig.method <- method
   if (!is.null(gr) && !is.character(gr)) { orig.gr <- gr1 } else {orig.gr <- gr }
@@ -114,6 +121,9 @@ optimr <- function(par, fn, gr=NULL, hess=NULL, method=NULL, lower=-Inf, upper=I
 
   efn <- function(spar) { # dotargs removed 230619
       optsp$kfn<-optsp$kfn+1 # counter
+# Emergency stop mechanism commented out as it will not stop gracefully from opm()
+#      cat("efn::control$maxfeval=",control$maxfeval," optsp$kfn=",optsp$kfn,"\n")
+#      if (optsp$kfn > 2 * control$maxfeval) stop("Emergency stop on function evaluation count")
       # rely on pscale being defined in this enclosing environment
       par <- spar*pscale
       val <- fn1(par) * fnscale
@@ -125,7 +135,7 @@ optimr <- function(par, fn, gr=NULL, hess=NULL, method=NULL, lower=-Inf, upper=I
      if ( ! (gr %in% control$grapprox)) stop(gr," is not a valid gradient approximation code")
      if (control$trace>0) cat("Using numerical approximation '",gr,"' to gradient in optimr()\n")
      egr <- function(spar){ 
-       optsp$kgr<-optsp$kgr+1 # counter
+       optsp$kgr<-optsp$kgr+1 # counter # 20241202 -- was missing from scounts
        if (control$trace > 2) {
          cat("par:"); print(par)
          cat("fnscale =",fnscale,"  pscale="); print(pscale)
@@ -140,6 +150,9 @@ optimr <- function(par, fn, gr=NULL, hess=NULL, method=NULL, lower=-Inf, upper=I
     else {
        egr <- function(spar) {
          optsp$kgr<-optsp$kgr+1 # counter
+# Emergency stop mechanism commented out as it will not stop gracefully from opm()
+#         cat("egr::control$maxit=",control$maxit," optsp$kgr=",optsp$kgr,"\n")
+#         if (optsp$kgr > 2 * control$maxit) stop("Emergency stop on gradient evaluation count")
          par <- spar*pscale
          result <- gr1(par) * pscale * fnscale
        }
@@ -286,13 +299,12 @@ optimr <- function(par, fn, gr=NULL, hess=NULL, method=NULL, lower=-Inf, upper=I
             if(control$trace > 3) cat("L-BFGS-B start: ")
             ans <- try(optim(par=par, fn=efn, gr=egr, 
                       lower=lower, upper=upper, method="L-BFGS-B", hessian=FALSE, 
-                       control=mcontrol, ...))
+                       control=mcontrol)) # remove , ... 231016
             if(control$trace > 3) cat(" ans$value=",ans$value,"\n")
           }
         } else {
           ans <- try(optim(par=par, fn=efn, gr=egr, 
-                method=method, hessian=FALSE, control=mcontrol))
-#                method=method, hessian=FALSE, control=mcontrol, ...))
+                method=method, hessian=FALSE, control=mcontrol)) # remove , ... 231016
         }
         if (inherits(ans,"try-error")) { # bad result -- What to do?
 		  ans<-list() # ans not yet defined, so set as list
@@ -325,7 +337,6 @@ optimr <- function(par, fn, gr=NULL, hess=NULL, method=NULL, lower=-Inf, upper=I
         ## if(nsctrl > 0) { stop("There are no extra controls set up for ",method) }
         ans <- try(nlminb(start=spar, objective=efn, gradient=egr, hessian=ehess, lower=slower, 
 		upper=supper, control=mcontrol))
-#		upper=supper, control=mcontrol,  ...))
         if (! inherits(ans, "try-error")) {
 		# Translate output to common format and names
         	ans$value<-ans$objective
@@ -403,6 +414,8 @@ optimr <- function(par, fn, gr=NULL, hess=NULL, method=NULL, lower=-Inf, upper=I
         mcontrol$trace <- control$trace
         mcontrol$maxit <- control$maxit # 151217 JN
         mcontrol$stepredn <- control$stepredn # 220217
+        mcontrol$maxfeval <- control$maxfeval # 241023
+#        print(mcontrol)
         ## if(nsctrl > 0) { stop("There are no extra controls set up for ",method) }
         if (! is.null(egr)) { #?? why not slower, supper?
    	     ans <- try(ncg(par=spar, fn=efn, gr=egr, bds=bdmsk, control=mcontrol))
@@ -436,6 +449,7 @@ optimr <- function(par, fn, gr=NULL, hess=NULL, method=NULL, lower=-Inf, upper=I
       else if (method == "Rcgmin") { # Use Rcgmin routine (ignoring masks)
         mcontrol$trace <- control$trace
         mcontrol$maxit <- control$maxit # 151217 JN
+        mcontrol$maxfeval <- control$maxfeval # 241023
         ## if(nsctrl > 0) { stop("There are no extra controls set up for ",method) }
         if (! is.null(egr)) {
   	  if (control$have.bounds) { # 151220 -- this was not defined
@@ -477,6 +491,7 @@ optimr <- function(par, fn, gr=NULL, hess=NULL, method=NULL, lower=-Inf, upper=I
         mcontrol$trace <- control$trace
         mcontrol$maxit <- control$maxit # 151217 JN
         mcontrol$stepredn <- control$stepredn # 220217
+        mcontrol$maxfeval <- control$maxfeval # 241023
         ## if(nsctrl > 0) { stop("There are no extra controls set up for ",method) }
         if (! is.null(egr)) { #?? why not slower, supper?
 # 230622 update to simplify call
@@ -616,7 +631,7 @@ optimr <- function(par, fn, gr=NULL, hess=NULL, method=NULL, lower=-Inf, upper=I
          ## return(ans)
       }  ## end if using snewton
   ## --------------------------------------------
-  else if (method == "snewtonm") { # Use snewtonm routine
+  else if ((method == "snewtm") || (method=="snewtonm")) { # Use snewtm/snewtonm routine
     mcontrol$maxit <- control$maxit
     mcontrol$maxfeval <- control$maxfeval # changed from maxfevals 180321
     mcontrol$trace <- control$trace # 140902 Note no check on validity of values
@@ -686,7 +701,7 @@ optimr <- function(par, fn, gr=NULL, hess=NULL, method=NULL, lower=-Inf, upper=I
       }
     } # end have answer
     ## return(ans)
-  }  ## end if using snewtonm
+  }  ## end if using snewtm
   ## --------------------------------------------
       else if (method == "hjn") {# Use JN Hooke and Jeeves
         if (control$trace > 1) { 
@@ -815,6 +830,9 @@ optimr <- function(par, fn, gr=NULL, hess=NULL, method=NULL, lower=-Inf, upper=I
         errmsg <- NA
         class(ans)[1] <- "undefined" # initial setting
         ## if(nsctrl > 0) { stop("There are no extra controls set up for ",method) }
+## Count limits NOT working for Rtnmin
+##        mcontrol$maxit <- control$maxit
+##        mcontrol$maxfun <- control$maxfevals        
         if (is.null(egr)) { ## fixed msg below (referred to lbfgs) 170214
             if (control$trace > 0) cat("Rtnmin MUST have gradient provided\n")
             errmsg <- "Rtnmin MUST have gradient provided"
@@ -822,9 +840,9 @@ optimr <- function(par, fn, gr=NULL, hess=NULL, method=NULL, lower=-Inf, upper=I
         } else {
            if (control$have.bounds) {
    	      ans <- try(tnbc(x=spar, fgfun=fgfn, lower=slower,
-                   upper=supper, trace=mcontrol$trace))
+                   upper=supper, trace=mcontrol$trace, control=mcontrol))
            } else {
-   	      ans <- try(tn(x=spar, fgfun=fgfn, trace=mcontrol$trace))
+   	      ans <- try(tn(x=spar, fgfun=fgfn, trace=mcontrol$trace, control=mcontrol))
 	   }
         }
         if (inherits(ans,"try-error")) {
@@ -1117,6 +1135,7 @@ optimr <- function(par, fn, gr=NULL, hess=NULL, method=NULL, lower=-Inf, upper=I
 ## --------------------------------------------
       else if (method == "lbfgsb3c") {# Use 2011 L-BFGS-B wrapper
         if (control$trace > 1) cat("lbfgsb3c\n")
+        mcontrol$maxit <- control$maxit # 151217 JN
         mcontrol$trace <- control$trace
 # 170924 no longer needed
 ##        if (control$trace < 1) {mcontrol$iprint <- -1} else {mcontrol$iprint <- control$trace} 
@@ -1141,6 +1160,7 @@ optimr <- function(par, fn, gr=NULL, hess=NULL, method=NULL, lower=-Inf, upper=I
  ## Need to check these carefully. Changed 20191202 for lbfgsb3c !!?
             ans$par <- ans$par*pscale
             ans$niter <- NULL # loss of information
+            if (is.na(ans$convergence)){ ans$convergence <- 9995 }
          } else {
             if (control$trace>0) cat("lbfgsb3c failed for current problem \n")
             ans<-list(fevals=NA) # ans not yet defined, so set as list
@@ -1150,7 +1170,7 @@ optimr <- function(par, fn, gr=NULL, hess=NULL, method=NULL, lower=-Inf, upper=I
             ans$counts[1] <- NA
             ans$counts[1] <- NA
          }
-         ## return(ans)
+#         return(ans)
       }  ## end if using lbfgsb3c
 ## --------------------------------------------
       else if (method == "lbfgs") {# Use unconstrained method from lbfgs package
@@ -1174,12 +1194,12 @@ optimr <- function(par, fn, gr=NULL, hess=NULL, method=NULL, lower=-Inf, upper=I
             class(ans)[1] <- "try-error"            
         }
         ## if(nsctrl > 0) { stop("There are no extra controls set up for ",method) }
-        if (inherits(ans, "undefined")){ #?? need to explain why no dots??
-            ans <- try(lbfgs::lbfgs(efn, egr, vars=spar, 
+        if (inherits(ans, "undefined")){ 
+            ans <- try(lbfgs::lbfgs(efn, egr, vars=spar, max_iterations=control$maxit,
                     invisible=invisible))
         }
         if (! inherits(ans, "try-error")) {
-        ## Need to check these carefully!!?
+        ## Need to check these carefully!!??
             ans$par <- ans$par*pscale
             ans$counts[1] <- NA # lbfgs seems to have no output like this
             ans$counts[2] <- NA
@@ -1342,9 +1362,7 @@ optimr <- function(par, fn, gr=NULL, hess=NULL, method=NULL, lower=-Inf, upper=I
       }  ## end if using slsqp
 ## --------------------------------------------
   else if (method == "tnewt") {# tnewton from nloptr
-      if (control$trace > 1) cat("tnewt\n")
-      # Following seems to be needed to avoid unwanted output
-      #  if (control$trace < 1) {invisible <- 1} else {invisible <- 0}
+      if (control$trace > 1) cat("tnewt: Note that trace control not available for nloptr::tnewton\n")
       ans <- list() # to define the answer object
       errmsg <- NA
 #      nlalg <- "NLOPT_LD_SLSQP"
@@ -1352,11 +1370,10 @@ optimr <- function(par, fn, gr=NULL, hess=NULL, method=NULL, lower=-Inf, upper=I
       if (inherits(ans, "undefined")){
          dotstuff<-list(...)
          if (is.null(dotstuff)) { cat("No ...\n") }
-          maxeval<-control$maxfeval
           if (length(slower) == 1) slower<-rep(slower, npar)
           if (length(supper) == 1) supper<-rep(supper, npar)
           tans <- try(nloptr::tnewton(x0=spar, efn, egr, lower=slower, upper=supper,
-                        control=list(maxeval=maxeval)) )
+                        control=list(maxeval=control$maxfeval)) )
       }
       if (control$trace > 3) {
         cat("interim answer:")
@@ -1389,7 +1406,7 @@ optimr <- function(par, fn, gr=NULL, hess=NULL, method=NULL, lower=-Inf, upper=I
          }
       }  ## end if using tnewton
 ## --------------------------------------------
-  else if (method == "anms") {# Use unconstrained method from marqLevAlg
+  else if (method == "anms") {# Use nelder-mead method from pracma
       if (control$trace > 1) cat("anms\n")
       # Following seems to be needed to avoid unwanted output
       #  if (control$trace < 1) {invisible <- 1} else {invisible <- 0}
@@ -1548,13 +1565,14 @@ optimr <- function(par, fn, gr=NULL, hess=NULL, method=NULL, lower=-Inf, upper=I
       else {
          attr(ans$par, "status") <- rep(" ",npar)
          attr(ans$value, "ptype") <- "U"
-      }         
+      }
+      names(ans$par) <- parnam # restore the names
       if (savehess) { # compute hessian
          if (is.null(orig.hess)){
            if (is.null(orig.gr) || is.character(orig.gr) ) {
-              hes <- hesf(orig.fn, ans$par, ...) # from numDeriv or pracma??
+              hes <- hesf(fn1, ans$par) # from numDeriv or pracma, fix 20250403 (fn1)
            } else { 
-             hes <- jacf(orig.gr, ans$par, ...) # use Jacobian of gradient
+             hes <- jacf(gr1, ans$par) # use Jacobian of gradient, change to gr1 20250403
              # 20230613: this approach may give asymmetry of matrix
              hes <- 0.5*(hes + t(hes)) # symmetrize
            }
@@ -1564,5 +1582,6 @@ optimr <- function(par, fn, gr=NULL, hess=NULL, method=NULL, lower=-Inf, upper=I
          }
       } else { hes <- NULL } # to ensure it is defined
       ans$hessian <- hes
+      attr(ans, "maximize") <- control$maximize # added 241224
       ans # last statement of routine
 } ## end of optimr
